# Copyright (c) 2003-2004 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

__author__ = "Jean-Marc Orliaguet <jmo@ita.chalmers.se>"

"""
  BaseTemplet
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from OFS.PropertyManager import PropertyManager
from DateTime import DateTime
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName

from CPSSkinsPermissions import ManageThemes
from cpsskins_utils import rebuild_properties, callAction, \
                           getFreeId, verifyThemePerms, \
                           getObjectVisibility, canonizeId, \
                           getApplicableStylesFor, getStyleList, \
                           getAvailableLangs, getDefaultLang


def addBaseTemplet(dispatcher, id, REQUEST=None):
    """Add a Base Templet."""
    ob = BaseTemplet(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)

factory_type_information = (
    {'id': 'Base Templet',
     'description': ('A Base Templet is the most basic templet.'),
     'meta_type': 'Base Templet',
     'icon': 'templet_icon.gif',
     'product': 'CPSSkins',
     'factory': 'addBaseTemplet',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'actions': (
                 {'id': 'view',
                  'name': 'View',
                  'action': 'cpsskins_templet_view',
                  'category': 'object',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'cpsskins_edit_form',
                  'visible': 1,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'moveleft',
                  'name': 'Move Left',
                  'action': 'cpsskins_templet_moveleft',
                  'category': 'object',
                  'visible': 0,
                  'permissions': (ManageThemes,)},
                 {'id': 'moveright',
                  'name': 'Move Right',
                  'action': 'cpsskins_templet_moveright',
                  'visible': 0,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'moveup',
                  'name': 'Move Up',
                  'action': 'cpsskins_templet_moveup',
                  'visible': 0,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'movedown',
                  'name': 'Move Down',
                  'action': 'cpsskins_templet_movedown',
                  'visible': 0,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'toggle',
                  'name': 'Toggle',
                  'action': 'cpsskins_templet_toggle',
                  'visible': 0,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'delete',
                  'name': 'Delete',
                  'action': 'cpsskins_object_delete',
                  'visible': 0,
                  'category': 'object',
                  'permissions': (ManageThemes,)},
                 {'id': 'isportaltemplet',
                  'name': 'isportaltemplet',
                  'action': 'isportaltemplet',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class BaseTemplet(PortalContent, PropertyManager):
    """
    Base class for templets.
    """

    meta_type = None
    portal_type = None

    isportaltemplet = 1

    manage_options = ( PropertyManager.manage_options     # Properties
                     + PortalContent.manage_options[2:3]  # View
                     )

    security = ClassSecurityInfo()
    _actions = factory_type_information[0]['actions']

    _properties = (
        {'id': 'title', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Title', 
         'category': 'general'
        },
        {'id': 'closed', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Closed', 
         'category': 'none',
         'default': 0,
        },
        {'id': 'cacheable', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Cacheable', 
         'category': 'general',
         'default': 0
        },
        {'id': 'hidden_in_text_mode', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Hidden in text mode', 
         'category': 'visibility',
         'default': 0
        },
        {'id': 'display_title_in_text_mode', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Display title in text mode', 
         'category': 'visibility',
         'default': 1
        },
        {'id': 'xpos', 
         'type': 'int', 
         'mode': 'w', 
         'label': 'xpos',  
         'category': 'none'
        },
        {'id': 'align', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Horizontal alignment', 
         'select_variable': 'AlignList', 
         'category': 'layout',
         'visible': 'IsAlignable',
         'i18n': 1,
        },
        {'id': 'padding', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet padding', 
         'category': 'layout'
        },
        {'id': 'margin', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet margin',  
         'category': 'layout'
        },
        {'id': 'templet_height', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet height', 
         'category': 'layout'
        },
        {'id': 'fontcolor', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Font color', 
         'select_variable': 'FontColorsList', 
         'meta_type': 'Font Color', 
         'category': 'style'
        },
        {'id': 'fontshape', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Font shape', 
         'select_variable': 'FontShapesList', 
         'meta_type': 'Font Shape', 
         'category': 'style'
        },
        {'id': 'shape', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Templet shape', 
         'select_variable': 'AreaShapesList', 
         'meta_type': 'Area Shape', 
         'category': 'style'
        },
        {'id': 'color', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Templet color', 
         'select_variable': 'AreaColorsList', 
         'meta_type': 'Area Color',  
         'category': 'style'
        },
        {'id': 'visibility',  
         'type': 'selection',  
         'mode': 'w', 
         'label': 'Visibility criteria', 
         'select_variable': 'VisibilityList', 
         'category': 'visibility',
         'default': 'always',
         'i18n': 1,
        },
        {'id': 'visibility_paths',  
         'type': 'multiple selection',   
         'mode': 'w', 
         'label': 'The visibility paths', 
         'select_variable': 'cpsskins_listPaths', 
         'category': 'visibility',
         'visible': 'ShowVisibilityPaths'
        },
        {'id': 'languages', 
         'type': 'multiple selection', 
         'mode': 'w', 
         'label': 'The languages in which it is visible', 
         'select_variable': 'LanguagesList', 
         'category': 'visibility',
         'visible': 'LanguagesList',
        },
     )


    def __init__(self, id, 
                       title = 'Templet',
                       closed = 0,
                       cacheable = 0,
                       hidden_in_text_mode = 0,
                       display_title_in_text_mode = 0,
                       visibility = 'always',
                       visibility_paths= [],
                       languages = [],
                       xpos = 0, 
                       align = 'left', 
                       templet_height = '',
                       padding = '0.5em', 
                       margin = '0.5em', 
                       fontshape = 'Arial',
                       fontcolor = 'Black',
                       shape = 'NoBorder',
                       color = 'Transparent',
                       **kw):
        self.id = id
        self.closed = closed
        self.cacheable = cacheable
        self.hidden_in_text_mode = hidden_in_text_mode
        self.display_title_in_text_mode = display_title_in_text_mode
        self.visibility = visibility
        self.visibility_paths = visibility_paths
        self.languages = languages
        self.title = title
        self.xpos = int(xpos)
        self.align = align
        self.templet_height = templet_height
        self.padding = padding
        self.margin = margin
        self.fontshape = fontshape
        self.fontcolor = fontcolor
        self.shape = shape
        self.color = color

    security.declarePublic('isPortalTemplet')
    def isPortalTemplet(self):
        """ is portal templet """
           
        return 1

    security.declarePublic('isPortalContent')
    def isPortalContent(self):
        """ is portal content """
           
        return None

    security.declarePublic('ShowVisibilityPaths')
    def ShowVisibilityPaths(self):
        """ returns true if the visibility paths must be shown """
           
        category = getattr(self, 'visibility', None)
        if category in ['everywhere_except_in', \
                        'only_in', \
                        'starting_from', \
                        'up_till']:
            return 1

    security.declarePublic('isPortalBox')
    def isPortalBox(self):
        """ is portal box """
           
        return None

    security.declarePublic('isPortalBoxGroup')
    def isPortalBoxGroup(self):
        """ is portal box group """
           
        return None

    security.declarePublic('isAlignable')
    def isAlignable(self):
        """ Returns true if the Templet can be aligned horizontally """

        return 1

    security.declarePublic('AlignList')
    def AlignList(self):
        """ Returns a list of alignments for this object"""

        list = ['left', 'center', 'right']
        return list


    security.declarePublic('VisibilityList')
    def VisibilityList(self):
        """ Returns a list of visibility criteria """

        list = ['always', 
                'everywhere_except_in', 
                'only_in', 
                'starting_from', 
                'up_till', 
                'if_authenticated', 
                'if_anonymous', 
                'if_secure_connection' ]
        return list


    security.declarePublic('getTitle')
    def getTitle(self):
        """
        Gets the templet's title
        """

        return getattr(self, 'title', None)

    security.declareProtected(ManageThemes, 'getTempletPosition')
    def getTempletPosition(self):
        """ 
        Return the page block's ypos in the theme folder
        """

        container = self.aq_parent
        return container.get_object_position(self.getId())


    security.declarePublic('getI18nProperties')
    def getI18nProperties(self):
        """ Returns the list of i18n properties """

        return []


    security.declarePublic('getIconRelative')
    def getIconRelative(self):
        """
        Gets the Icon path, relative to the portal.
        This is needed to catalog correctly in the presence of VHM.
        """
        return self.getIcon(1)

    security.declareProtected(ManageThemes, 'rebuild')
    def rebuild(self, **kw):
        """                  
        Rebuild this templet
        """              

        setperms = kw.get('setperms', 0)
        canonizeId(self)
        rebuild_properties(self)
        if setperms:
            verifyThemePerms(self)
 

    security.declarePublic('getVisibility')
    def getVisibility(self, REQUEST=None):
        """ Return 1 if the Templet is visible """

        if self.closed:
            return None

        return getObjectVisibility(self, REQUEST)

    security.declarePublic('getInfo')
    def getInfo(self):
        """ Returns action information about this Templet """

        infoblock = {}
        actions_list = ['moveup', 'movedown', 'moveleft', 'moveright', 'toggle', 'delete']

        ti = self.getTypeInfo()
        for actionid in actions_list:  
             actioninfo = {}
             if actionid == 'moveup':
                 actioninfo['can_move'] = self.can_moveup()
             elif actionid == 'moveleft':
                 actioninfo['can_move'] = self.can_moveleft()
             elif actionid == 'moveright':
                 actioninfo['can_move'] = self.can_moveright()
             elif actionid == 'movedown':
                 actioninfo['can_move'] = self.can_movedown()
             elif actionid == 'toggle':
                 actioninfo['can_toggle'] = self.can_toggle()
             elif actionid == 'delete':
                 actioninfo['can_delete'] = self.can_delete()
             try:
                action = ti.getActionById(actionid)
                actioninfo['url']  = self.absolute_url() + '/' + self.restrictedTraverse(action).getId()
             except:
                continue
             infoblock[actionid] = actioninfo
                            
        return infoblock 

    security.declarePublic('getApplicableStyles')
    def getApplicableStyles(self):
        """ Returns the styles by meta type that are 
            applicable to this Templet
        """
        return getApplicableStylesFor(self)

    security.declareProtected(ManageThemes, 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return callAction(self, 'edit', **kw)

    security.declareProtected(ManageThemes, 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """
        self.manage_changeProperties(**kw)
        

    security.declarePublic('moveleft_pos')
    def moveleft_pos(self):
        """
        Can the templet be moved to the left?
        """

        if self.can_moveleft():
           return self.xpos -1 
        return 


    security.declarePublic('moveright_pos')
    def moveright_pos(self):
        """
        Return the new position 
        """

        if self.can_moveright():
            return self.xpos + 1
        return 

    security.declarePublic('moveup_pos')
    def moveup_pos(self):
        """
        Return the new position 
        """
        container = self.aq_parent
        this_pos = container.get_object_position(self.getId())
        newpos = -1
        for obj in container.objectValues():
            o = obj.aq_explicit
            if hasattr(o, 'isPortalTemplet'):
               if o.isPortalTemplet():
                    if obj.xpos == self.xpos:
                        pos = container.get_object_position(obj.getId())
                        if pos > newpos and pos < this_pos:
                            newpos = pos
        return newpos

    security.declarePublic('movedown_pos')
    def movedown_pos(self):
        """
        Return the new position 
        """
        container = self.aq_parent
        this_pos = container.get_object_position(self.getId())
        newpos = -1
        for obj in container.objectValues():
            o = obj.aq_explicit
            if hasattr(o, 'isPortalTemplet'):
                if o.isPortalTemplet():
                    if obj.xpos == self.xpos:
                        pos = container.get_object_position(obj.getId())
                        if  pos > this_pos:
                           newpos = pos
                           break
        return newpos


    security.declarePublic('can_toggle')
    def can_toggle(self):
        """
        Can the templet be toggled ( open/close ) ?
        """
        if hasattr(self, 'closed'):
            return 1


    security.declarePublic('can_delete')
    def can_delete(self):
        """
        Can the templet be deleted ?
        """
        return 1


    security.declarePublic('can_moveup')
    def can_moveup(self):
        """
        Can the templet be moved up?
        """

        # find a page block above it
        container = self.aq_parent
        if container.can_moveup():
            return 1

        # otherwise find a templet above it
        this_pos = container.get_object_position(self.getId())
        for obj in container.objectValues():
            o = obj.aq_explicit
            if hasattr(o, 'isPortalTemplet'):
               if o.isPortalTemplet():
                   if obj.xpos == self.xpos:
                       pos = container.get_object_position(obj.getId())
                       if pos < this_pos:
                           return 1
        return

    security.declarePublic('can_movedown')
    def can_movedown(self):
        """
        Can the templet be moved down?
        """

        container = self.aq_parent
        if container.can_movedown():
            return 1

        container = self.aq_parent
        this_pos = container.get_object_position(self.getId())
        for obj in container.objectValues():
            o =  obj.aq_explicit
            if hasattr(o, 'isPortalTemplet'):
                if o.isPortalTemplet():
                    if obj.xpos == self.xpos:
                        pos = container.get_object_position(obj.getId())
                        if pos > this_pos:
                            return 1
        return


    security.declarePublic('can_moveright')
    def can_moveright(self):
        """
        Can the templet be moved to the right?
        """

        container = self.aq_parent 
        maxcols = getattr(container, 'maxcols', None)
        if maxcols is None:
            return 
        if self.xpos < maxcols -1 :
            return 1
        return 


    security.declarePublic('can_moveleft')
    def can_moveleft(self):
        """
        Can the templet be moved to the left?
        """

        if self.xpos > 0:
            return 1
        return 

    security.declareProtected(ManageThemes, 'toggle')
    def toggle(self, state):
        """
        Open / close the templet
        """
        self.closed = state
        return 


    security.declareProtected(ManageThemes, 'duplicate')
    def duplicate(self):
        """
        Duplicate a Templet
        """
        
        container = self.aq_parent
        src_ypos = container.get_object_position(self.getId())
        newid = getFreeId(container)
        container.manage_clone(self, newid)
        newobj  = getattr(container, newid, None)
        container.move_object_to_position(newobj.getId(), src_ypos + 1)

        # needed for the ImageBox 
        setattr(newobj, 'id', newid)
        verifyThemePerms(newobj)
        return newobj


    security.declareProtected(ManageThemes, 'set_property')
    def set_property(self, prop=None, value=None):
        """
        Sets a property 
        """

        if value is None:
           return
     
        if not hasattr(self, prop):
           return
        setattr(self, prop, value)


    security.declareProtected(ManageThemes, 'move_to_pageblock')
    def move_to_pageblock(self, pageblock=None, xpos=None, ypos=None, REQUEST=None):
        """
        Move the templet to a given pageblock 
        returns the instance of the object moved
        """

        if pageblock is None:
            return
        if xpos is None:
            return
        if ypos is None:
            return
        id = self.getId()
        current_xpos = self.xpos
        new_xpos = int(xpos)
        self.xpos = new_xpos
        container = self.aq_parent
        src_pageblock = container.getId()
        theme_container = container.aq_parent
        current_ypos = container.get_object_position(self.getId())

        if pageblock == container.getId() and \
                        int(xpos) == current_xpos and \
                        int(ypos) == current_ypos:
            return

        for obj in theme_container.objectValues():
           if obj.getId() == pageblock:
               cookie = container.manage_cutObjects(self.getId(), REQUEST=REQUEST)
               res = obj.manage_pasteObjects(cookie) 
               new_id = res[0]['new_id']
               newpos = int(ypos) 
               if pageblock == src_pageblock:
                    if newpos > current_ypos and new_xpos != current_xpos:
                        newpos = newpos -1;
               obj.move_object_to_position(new_id, newpos)
               templet = getattr(obj, new_id)
               return templet


    security.declareProtected(ManageThemes, 'copy_to_theme')
    def copy_to_theme(self, dest_theme=None, REQUEST=None):
        """
        Copies the Templet to another theme.
        The copied object is placed into the first available Page Block.
        returns the copied object
        """

        if dest_theme is None:
            return
        tmtool = getToolByName(self, 'portal_themes')
        container = self.aq_parent
        dest_theme_container = tmtool.getThemeContainer(theme=dest_theme)
        if dest_theme_container is None:
            return

        pageblocks = dest_theme_container.getPageBlocks()
        if pageblocks:
            dest_container = pageblocks[0]
        else:
            return

        cookie = container.manage_copyObjects(self.getId(), REQUEST=REQUEST)
        res = dest_container.manage_pasteObjects(cookie) 
        new_id = res[0]['new_id']
        templet = getattr(dest_container, new_id)
        verifyThemePerms(templet)
        templet.xpos = 0
        templet.ypos = 0
        return templet



    security.declareProtected(ManageThemes, 'move')
    def move(self, direction=None, REQUEST=None):
        """
        Move the templet into a direction 
        returns the instance of the moved object
        """
  
        container = self.aq_parent 
        if direction == 'left' and self.can_moveleft():
            self.xpos = self.moveleft_pos()
            return self
 
        if direction == 'right' and self.can_moveright():
            self.xpos = self.moveright_pos()
            return self

        if direction == 'up' and self.can_moveup():
            newpos = self.moveup_pos()
            if newpos >= 0:
                container.move_object_to_position(self.getId(), newpos)
                return self
            else:
                cookie = container.manage_cutObjects(self.getId(), REQUEST=REQUEST)
                pos =  container.moveup_pos()
                theme_container = container.aq_parent
                for obj in theme_container.objectValues():
                    if theme_container.get_object_position(obj.getId()) == pos:
                        obj.manage_pasteObjects(cookie)
                        maxcols = getattr(obj, 'maxcols', None)
                        if maxcols is not None and self.xpos > maxcols-1:
                             self.xpos = maxcols - 1
                        return getattr(obj, self.getId(), None) 

        if direction == 'down' and self.can_movedown():
            newpos = self.movedown_pos()
            if newpos >= 0:
                container.move_object_to_position(self.getId(), newpos)
                return self
            else:
                cookie = container.manage_cutObjects(self.getId(), REQUEST=REQUEST)
                pos =  container.movedown_pos()
                theme_container = container.aq_parent
                for obj in theme_container.objectValues():
                    if theme_container.get_object_position(obj.getId()) == pos:
                        obj.manage_pasteObjects(cookie)
                        newobj =  getattr(obj, self.getId(), None) 
                        obj.move_object_to_position(newobj.getId(), int(0))

                        maxcols = getattr(obj, 'maxcols', None)
                        if maxcols is not None and self.xpos > maxcols-1:
                             self.xpos = maxcols - 1

                        return newobj
        return 


    security.declareProtected(ManageThemes, 'moveleft')
    def moveleft(self):
        """
        Move the templet to the left
        returns the instance of the moved object
        """

        return self.move(direction='left')


    security.declareProtected(ManageThemes, 'moveright')
    def moveright(self):
        """
        Move the templet to the right 
        returns the instance of the moved object
        """

        return self.move(direction='right')


    security.declareProtected(ManageThemes, 'moveup')
    def moveup(self):
        """
        Move the templet upwards
        returns the instance of the moved object
        """

        return self.move(direction='up')


    security.declareProtected(ManageThemes, 'movedown')
    def movedown(self):
        """
        Move the templet downwards
        returns the instance of the moved object
        """

        return self.move(direction='down')


    security.declareProtected(ManageThemes, 'change_alignment')
    def change_alignment(self, alignment=None):
        """
        Align the templet 
        """

        if alignment in ['left', 'center', 'right']:
           self.set_property('align', alignment)


    security.declarePublic('render')
    def render(self, **kw):
        """
        Renders the templet.
        """

        fail = 0
        if hasattr(self.aq_explicit, 'render_action'):
            actionid = self.render_action
        else:
            ti = self.getTypeInfo() 
            try: 
                actionid = ti.getActionById('render')
            except:
                fail = 1

        if not fail:
            meth = getattr(self, actionid, None)
            if meth is not None:
                rendered = apply(meth, (), kw)
            else:
                fail = 1

        if fail:
            rendered = self.cpsskins_brokentemplet()
        return rendered


    security.declarePublic('AreaColorsList')
    def AreaColorsList(self):           
        """ Returns a list of Area Color styles"""

        return getStyleList(self, 'Area Color')


    security.declarePublic('AreaShapesList')
    def AreaShapesList(self):           
        """ Returns a list of Area Shape styles"""

        return getStyleList(self, 'Area Shape')


    security.declarePublic('FontColorsList')
    def FontColorsList(self):           
        """ Returns a list of Font Color styles"""

        return getStyleList(self, 'Font Color')


    security.declarePublic('FontShapesList')
    def FontShapesList(self):           
        """ Returns a list of Font Shape styles"""

        return getStyleList(self, 'Font Shape')


    security.declarePublic('LanguagesList')
    def LanguagesList(self):           
        """ Returns a list of languages """

        return getAvailableLangs(self)


    security.declarePublic('getDefaultLang')
    def getDefaultLang(self):
        """ returns the code name of the default language """
           
        return getDefaultLang(self)

InitializeClass(BaseTemplet)


